package org.byteam.dfa;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.SharedPreferences;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import org.byteam.dfa.model.Patch;

/**
 * 在加载patch的情况下，设置一个handler用于捕获异常
 */
public class CrashHandler implements Thread.UncaughtExceptionHandler {
    private static final String TAG = "Dfa.CrashHandler";
    private Context mContext;

    @Nullable
    private Thread.UncaughtExceptionHandler mOriginHandler;

    private static CrashHandler mInstance;

    private CrashHandler() {
    }

    public static synchronized CrashHandler getInstance() {
        if (mInstance == null) {
            mInstance = new CrashHandler();
        }
        return mInstance;
    }

    public void init(Context context) {
        if (context.getApplicationContext() != null) {
            this.mContext = context.getApplicationContext();
        } else {
            this.mContext = context;
        }
        mOriginHandler = Thread.currentThread().getUncaughtExceptionHandler();
        if (mOriginHandler == null) {
            mOriginHandler = Thread.getDefaultUncaughtExceptionHandler();
        }
        Thread.currentThread().setUncaughtExceptionHandler(this);
    }

    @SuppressLint("ApplySharedPref")
    @Override
    public void uncaughtException(@NonNull Thread t, @NonNull Throwable e) {
        Patch patch = DfaLoader.getLoadedPatch(mContext);
        if (patch != null) {
            SharedPreferences sp = DfaLoader.getSharedPreferences(mContext);
            String key = patch.getAppVersionCode() + "_" + patch.getVersion() + "_crash_counter";
            int crashCounter = sp.getInt(key, 0);
            Log.w(TAG, "uncaughtException: crashCounter=" + crashCounter);
            sp.edit().putInt(key, ++crashCounter).commit();
            if (crashCounter >= 3) {
                Log.w(TAG, "uncaughtException: deletePatch");
                DfaLoader.deletePatch(patch);
            }
        }

        if (mOriginHandler != null) {
            mOriginHandler.uncaughtException(t, e);
        }
    }
}
