package org.byteam.dfa;

import android.app.Activity;
import android.app.Application;

import androidx.annotation.NonNull;

import org.byteam.dfa.model.Patch;
import org.byteam.dfa.util.ResultWrapper;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

@SuppressWarnings("unused")
public class DfaPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
    private static final String TAG = "DfaPlugin";
    private Application application;
    private MethodChannel channel;
    private DfaDelegate delegate;
    private ActivityPluginBinding activityPluginBinding;
    private FlutterPluginBinding flutterPluginBinding;

    public static void registerWith(Registrar registrar) {
        final DfaPlugin instance = new DfaPlugin();
        Application application = null;
        if (registrar.activity() != null) {
            application = registrar.activity().getApplication();
        } else if (registrar.context() != null) {
            application = (Application) registrar.context().getApplicationContext();
        }
        Activity activity = registrar.activity();
        instance.setup(registrar.messenger(), application, activity, registrar, null);
    }

    private void setup(final BinaryMessenger messenger,
                       final Application application,
                       final Activity activity,
                       final Registrar registrar,
                       final ActivityPluginBinding activityPluginBinding) {
        this.application = application;
//        this.activity = activity;
        this.delegate = new DfaDelegate(activity);
        this.activityPluginBinding = activityPluginBinding;
        this.channel = new MethodChannel(messenger, "dfa");
        this.channel.setMethodCallHandler(this);
        if (registrar != null) {
            // V1 embedding setup.
            registrar.addActivityResultListener(this.delegate);
            registrar.addRequestPermissionsResultListener(this.delegate);
        } else if (activityPluginBinding != null) {
            // V2 embedding setup.
            activityPluginBinding.addActivityResultListener(this.delegate);
            activityPluginBinding.addRequestPermissionsResultListener(this.delegate);
        }
    }

    private void tearDown() {
        if (this.activityPluginBinding != null) {
            this.activityPluginBinding.removeActivityResultListener(this.delegate);
            this.activityPluginBinding.removeRequestPermissionsResultListener(this.delegate);
            this.activityPluginBinding = null;
        }
        this.delegate = null;
        this.channel.setMethodCallHandler(null);
        this.channel = null;
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        this.flutterPluginBinding = binding;
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        this.flutterPluginBinding = null;
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        this.activityPluginBinding = binding;
        this.setup(this.flutterPluginBinding.getBinaryMessenger(),
                (Application) this.flutterPluginBinding.getApplicationContext(),
                binding.getActivity(),
                null,
                binding);
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        this.onDetachedFromActivity();
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        this.onAttachedToActivity(binding);
    }

    @Override
    public void onDetachedFromActivity() {
        tearDown();
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result rawResult) {
        MethodChannel.Result result = new ResultWrapper(rawResult);
        if ("installPatch".equals(call.method)) { // 安装
            String path = call.argument("path");
            delegate.installPatch(application, path, result);
        } else if ("installedPatch".equals(call.method)) { // 已安装
            Patch patch = DfaLoader.getInstalledPatch(this.application);
            if (patch != null) {
                result.success(patch.toJsonString());
            } else {
                result.success(null);
            }
        } else if ("loadedPatch".equals(call.method)) { // 已加载
            Patch patch = DfaLoader.getLoadedPatch(this.application);
            if (patch != null) {
                result.success(patch.toJsonString());
            } else {
                result.success(null);
            }
        } else if ("cleanAll".equals(call.method)) {
            delegate.cleanAll();
            result.success(null);
        } else if ("choosePatchFile".equals(call.method)) {
            delegate.choosePatchFile(result);
        } else {
            result.notImplemented();
        }
    }
}
