package org.byteam.dfa.model;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Patch model.
 */
public class Patch {
    private JSONObject source;
    // original parts
    private String version;
    private long appVersionCode;
    private List<Asset> files;
    // new parts
    private String installPath;
    private long installTimestamp;
    // todo status ？

    public Patch(JSONObject json) {
        this.source = json;
        this.version = json.optString("version");
        this.appVersionCode = json.optLong("appVersionCode");

        this.installPath = json.optString("installPath");
        this.installTimestamp = json.optLong("installTimestamp");
        JSONArray addedArr = json.optJSONArray("files");
        if (addedArr != null) {
            files = new ArrayList<>(addedArr.length());
            for (int i = 0, len = addedArr.length(); i < len; i++) {
                JSONObject jsonObject = addedArr.optJSONObject(i);
                Asset item = new Asset(jsonObject);
                files.add(item);
            }
        }
    }

    public String getVersion() {
        return version;
    }

    public long getAppVersionCode() {
        return appVersionCode;
    }

    public List<Asset> getFiles() {
        return files;
    }

    public String getInstallPath() {
        return installPath;
    }

    public long getInstallTimestamp() {
        return installTimestamp;
    }

    public void setInstallPath(String installPath) {
        this.installPath = installPath;
    }

    public void setInstallTimestamp(long installTimestamp) {
        this.installTimestamp = installTimestamp;
    }

    public String toJsonString() {
        try {
            JSONObject jsonObject = new JSONObject(source.toString());
            jsonObject.put("installPath", installPath);
            jsonObject.put("installTimestamp", installTimestamp);
            return jsonObject.toString();
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return null;
    }
}
