import 'dart:io';

import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';

import 'dfa_manager.dart';
import 'patch.dart';

/// 用于加载patch包资源的AssetBundle
class DfaAssetBundle extends PlatformAssetBundle {
  String installPath;

  DfaAssetBundle() {
    WidgetsFlutterBinding.ensureInitialized();
    DfaManager.getLoadedPatch().then((Patch patch) {
      installPath = patch?.installPath;
    });
  }

  @override
  Future<ByteData> load(String key) async {
    // print('DfaAssetBundle#load: key = $key');
    if (Platform.isAndroid && installPath != null) {
      String path = '$installPath/assets/flutter_assets/$key';
      try {
        // print('DfaAssetBundle#load: path = $path');
        File assetFile = File(path);
        if (assetFile.existsSync()) {
          return assetFile.readAsBytesSync().buffer.asByteData();
        }
      } catch (e) {
        print('dfa#PatchAssetBundle#load: path = $path');
        print(e);
      }
    }
    return super.load(key);
  }
}

final AssetBundle dfaAssetBundle = DfaAssetBundle();
