import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';

import 'dfa_manager.dart';
import 'install_result.dart';
import 'patch.dart';

/// 开发者页面
class DevPage extends StatefulWidget {
  DevPage({Key key}) : super(key: key);

  @override
  _DevPageState createState() {
    return _DevPageState();
  }
}

class _DevPageState extends State<DevPage> {
  GlobalKey<ScaffoldState> _key = GlobalKey();
  bool _installing = false;
  Patch _loadedPatch;
  Patch _installedPatch;
  String _msg;

  @override
  void initState() {
    super.initState();
    _load();
  }

  @override
  void dispose() {
    super.dispose();
  }

  _load() async {
    _loadedPatch = await DfaManager.getLoadedPatch();
    _installedPatch = await DfaManager.getInstalledPatch();
    setState(() {});
  }

  _installPatch() async {
    if (_installing) {
      return;
    }
    String path;
    try {
      path = await DfaManager.choosePatchFile();
    } catch (e) {
      print(e);
      setState(() {
        _msg = e.toString();
      });
      return;
    }
    if (path == null) {
      setState(() {
        _msg = 'file path == null';
      });
      return;
    }

    File patchFile = File(path);
    if (!patchFile.existsSync()) {
      setState(() {
        _msg = '${patchFile.absolute.path} not exists.';
      });
      return;
    }
    _installing = true;
    InstallResult result = await DfaManager.installPatch(patchFile);
    _installing = false;
    _load();
    setState(() {
      _msg = '${result.toString()}';
    });
    if (result.success) {
      _key.currentState?.showSnackBar(SnackBar(
        content: Text('Successful! Takes effect after restart.'),
        action: SnackBarAction(
          label: 'Restart',
          onPressed: () {
            exit(0);
          },
        ),
      ));
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      key: _key,
      appBar: AppBar(
        title: Text('Patch Details'),
      ),
      body: Padding(
        padding: EdgeInsets.all(8),
        child: _buildTable(),
      ),
    );
  }

  Widget _buildTable() {
    return Table(
      border: TableBorder.all(),
      columnWidths: const <int, TableColumnWidth>{
        0: FlexColumnWidth(1),
        1: FlexColumnWidth(2),
      },
      children: [
        TableRow(
          children: [
            Padding(
              padding: const EdgeInsets.all(4),
              child: Text('Loaded patch:'),
            ),
            _buildPatchContent(_loadedPatch == null
                ? ''
                : JsonEncoder.withIndent('  ').convert(_loadedPatch?.source)),
          ],
        ),
        TableRow(
          children: [
            Padding(
              padding: const EdgeInsets.all(4),
              child: Text('Installed patch:'),
            ),
            _buildPatchContent(_installedPatch == null
                ? ''
                : JsonEncoder.withIndent('  ')
                    .convert(_installedPatch?.source)),
          ],
        ),
        TableRow(
          children: [
            Padding(
              padding: const EdgeInsets.all(4),
              child: Text('Install result:'),
            ),
            Padding(
              padding: const EdgeInsets.all(4),
              child: Text(
                _msg ?? '',
                style: TextStyle(
                  fontSize: 12,
                ),
              ),
            ),
          ],
        ),
        TableRow(
          children: [
            Padding(
              padding: const EdgeInsets.all(4),
              child: RaisedButton(
                child: Text('Install patch'),
                onPressed: _installPatch,
              ),
            ),
            Padding(
              padding: const EdgeInsets.all(4),
              child: RaisedButton(
                child: Text('Clean patch'),
                onPressed: () {
                  DfaManager.cleanAll();
                  _msg = '';
                  _load();
                },
              ),
            ),
          ],
        ),
      ],
    );
  }

  Widget _buildPatchContent(String content) {
    return ConstrainedBox(
      constraints: BoxConstraints(maxHeight: 200),
      child: Scrollbar(
        child: SingleChildScrollView(
          physics: ClampingScrollPhysics(),
          padding: EdgeInsets.all(4),
          child: Text(
            content ?? '',
            style: TextStyle(
              fontSize: 10,
            ),
          ),
        ),
      ),
    );
  }
}
